using MediatR;
using cuqmbr.TravelGuide.Application.Common.Interfaces.Persistence;
using cuqmbr.TravelGuide.Domain.Entities;
using AutoMapper;
using cuqmbr.TravelGuide.Application.Common.Exceptions;
using Microsoft.Extensions.Localization;
using FluentValidation.Results;

namespace cuqmbr.TravelGuide.Application.Routes.Commands.AddRoute;

public class AddRouteCommandHandler :
    IRequestHandler<AddRouteCommand, RouteDto>
{
    private readonly UnitOfWork _unitOfWork;
    private readonly IMapper _mapper;
    private readonly IStringLocalizer _localizer;

    public AddRouteCommandHandler(
        UnitOfWork unitOfWork,
        IMapper mapper,
        IStringLocalizer localizer)
    {
        _unitOfWork = unitOfWork;
        _mapper = mapper;
        _localizer = localizer;
    }

    public async Task<RouteDto> Handle(
        AddRouteCommand request,
        CancellationToken cancellationToken)
    {
        var page = await _unitOfWork.AddressRepository.GetPageAsync(
            e => request.Addresses.Select(a => a.Guid).Contains(e.Guid),
            e => e.City.Region.Country,
            1, request.Addresses.Count, cancellationToken);

        var invalidVehicleTypeAddress = 
            page.Items.FirstOrDefault(a => a.VehicleType != request.VehicleType);
        if (invalidVehicleTypeAddress != null)
        {
            throw new ValidationException(
                new List<ValidationFailure>
                {
                    new()
                    {
                        PropertyName = nameof(request.Addresses),
                        ErrorMessage = _localizer["Validation.SameVehicleType"]
                    }
                });
        }

        var pageContainsAllRequestedAddresses =
            request.Addresses.Select(e => e.Guid)
                .All(e => page.Items.Select(e => e.Guid).Contains(e));
        if (!pageContainsAllRequestedAddresses)
        {
            var notFoundCount = request.Addresses.Count - page.TotalCount;
            throw new NotFoundException(
                $"{notFoundCount} addresses was not found.");
        }

        var entity = new Route()
        {
            Name = request.Name,
            VehicleType = request.VehicleType,
            RouteAddresses = request.Addresses.Select(
                e => new RouteAddress()
                {
                    Order = e.Order,
                    AddressId = page.Items.Single(i => i.Guid == e.Guid).Id
                })
                .OrderBy(e => e.Order)
                .ToArray()
        };

        entity = await _unitOfWork.RouteRepository.AddOneAsync(
            entity, cancellationToken);

        await _unitOfWork.SaveAsync(cancellationToken);
        _unitOfWork.Dispose();

        return _mapper.Map<RouteDto>(entity);
    }
}
