using MediatR;
using cuqmbr.TravelGuide.Application.Common.Interfaces.Persistence;
using AutoMapper;
using cuqmbr.TravelGuide.Application.Common.Exceptions;
using FluentValidation.Results;
using Microsoft.Extensions.Localization;
using cuqmbr.TravelGuide.Domain.Entities;

namespace cuqmbr.TravelGuide.Application.Routes.Commands.UpdateRoute;

public class UpdateRouteCommandHandler :
    IRequestHandler<UpdateRouteCommand, RouteDto>
{
    private readonly UnitOfWork _unitOfWork;
    private readonly IMapper _mapper;
    public IStringLocalizer _localizer { get; set; }

    public UpdateRouteCommandHandler(
        UnitOfWork unitOfWork,
        IMapper mapper,
        IStringLocalizer localizer)
    {
        _unitOfWork = unitOfWork;
        _mapper = mapper;
        _localizer = localizer;
    }

    public async Task<RouteDto> Handle(
        UpdateRouteCommand request,
        CancellationToken cancellationToken)
    {
        var entity = await _unitOfWork.RouteRepository.GetOneAsync(
            e => e.Guid == request.Guid,
            e => e.RouteAddresses,
            cancellationToken);

        if (entity == null)
        {
            throw new NotFoundException();
        }

        var page = await _unitOfWork.AddressRepository.GetPageAsync(
            e => request.Addresses.Select(a => a.Guid).Contains(e.Guid),
            e => e.City.Region.Country,
            1, request.Addresses.Count, cancellationToken);

        var invalidVehicleTypeAddress =
            page.Items.FirstOrDefault(a => a.VehicleType != request.VehicleType);
        if (invalidVehicleTypeAddress != null)
        {
            throw new ValidationException(
                new List<ValidationFailure>
                {
                    new()
                    {
                        PropertyName = nameof(request.Addresses),
                        ErrorMessage = _localizer["Validation.SameVehicleType"]
                    }
                });
        }

        var pageContainsAllRequestedAddresses =
            request.Addresses.Select(e => e.Guid)
                .All(e => page.Items.Select(e => e.Guid).Contains(e));
        if (!pageContainsAllRequestedAddresses)
        {
            var notFoundCount = request.Addresses.Count - page.TotalCount;
            throw new NotFoundException(
                $"{notFoundCount} addresses was not found.");
        }


        entity.Guid = request.Guid;
        entity.Name = request.Name;
        entity.VehicleType = request.VehicleType;


        var requestRouteAddresses = request.Addresses.Select(
            e => new RouteAddress()
            {
                Order = e.Order,
                AddressId = page.Items.Single(i => i.Guid == e.Guid).Id
            });

        var commonRouteAddresses = entity.RouteAddresses.IntersectBy(
            requestRouteAddresses.Select(ra => (ra.Order, ra.AddressId)),
            ra => (ra.Order, ra.AddressId));

        var newRouteAddresses = requestRouteAddresses.ExceptBy(
            entity.RouteAddresses.Select(ra => (ra.Order, ra.AddressId)),
            ra => (ra.Order, ra.AddressId));

        var combinedRouteAddresses = commonRouteAddresses.UnionBy(
            newRouteAddresses, ra => (ra.Order, ra.AddressId));

        entity.RouteAddresses = combinedRouteAddresses
            .OrderBy(e => e.Order).ToList();


        entity = await _unitOfWork.RouteRepository.UpdateOneAsync(
            entity, cancellationToken);

        await _unitOfWork.SaveAsync(cancellationToken);
        _unitOfWork.Dispose();

        return _mapper.Map<RouteDto>(entity);
    }
}
