using MediatR;
using cuqmbr.TravelGuide.Application.Common.Interfaces.Persistence;
using AutoMapper;
using cuqmbr.TravelGuide.Application.Common.Models;
using cuqmbr.TravelGuide.Application.Common.Extensions;
using cuqmbr.TravelGuide.Domain.Entities;

namespace cuqmbr.TravelGuide.Application.Routes.Queries.GetRoutesPage;

public class GetRoutesPageQueryHandler :
    IRequestHandler<GetRoutesPageQuery, PaginatedList<RouteDto>>
{
    private readonly UnitOfWork _unitOfWork;
    private readonly IMapper _mapper;

    public GetRoutesPageQueryHandler(
        UnitOfWork unitOfWork,
        IMapper mapper)
    {
        _unitOfWork = unitOfWork;
        _mapper = mapper;
    }

    public async Task<PaginatedList<RouteDto>> Handle(
        GetRoutesPageQuery request,
        CancellationToken cancellationToken)
    {
        var paginatedList = await _unitOfWork.RouteRepository.GetPageAsync(
            e =>
                e.Name.ToLower().Contains(request.Search.ToLower()) &&
                (request.VehicleType != null
                    ? e.VehicleType == request.VehicleType
                    : true),
            e => e.RouteAddresses,
            request.PageNumber, request.PageSize,
            cancellationToken);

        foreach (var route in paginatedList.Items)
        {
            // TODO: Find a way to include through lists
            var addresses = await _unitOfWork.AddressRepository.GetPageAsync(
                e => route.RouteAddresses.Select(ra => ra.AddressId).Contains(e.Id),
                e => e.City.Region.Country,
                1, route.RouteAddresses.Count, cancellationToken);

            route.RouteAddresses = route.RouteAddresses.Select(
                e => new RouteAddress()
                {
                    Id = e.Id,
                    Guid = e.Guid,
                    Order = e.Order,
                    RouteId = e.RouteId,
                    Route = e.Route,
                    AddressId = e.AddressId,
                    Address = addresses.Items.First(a => a.Id == e.AddressId)
                })
                .OrderBy(e => e.Order)
                .ToArray();
        }

        var mappedItems = _mapper
            .ProjectTo<RouteDto>(paginatedList.Items.AsQueryable());

        mappedItems = QueryableExtension<RouteDto>
            .ApplySort(mappedItems, request.Sort);

        _unitOfWork.Dispose();

        return new PaginatedList<RouteDto>(
            mappedItems.ToList(),
            paginatedList.TotalCount, request.PageNumber,
            request.PageSize);
    }
}
