using Microsoft.AspNetCore.Mvc;
using Swashbuckle.AspNetCore.Annotations;
using cuqmbr.TravelGuide.Application.Common.Models;
using cuqmbr.TravelGuide.Application.Common.ViewModels;
using cuqmbr.TravelGuide.Domain.Enums;
using cuqmbr.TravelGuide.Application.Routes;
using cuqmbr.TravelGuide.Application.Routes.Commands.AddRoute;
using cuqmbr.TravelGuide.Application.Routes.Queries.GetRoutesPage;
using cuqmbr.TravelGuide.Application.Routes.Queries.GetRoute;
using cuqmbr.TravelGuide.Application.Routes.Commands.UpdateRoute;
using cuqmbr.TravelGuide.Application.Routes.Commands.DeleteRoute;
using cuqmbr.TravelGuide.Application.Routes.ViewModels;
using cuqmbr.TravelGuide.Application.Routes.Models;

namespace cuqmbr.TravelGuide.HttpApi.Controllers;

[Route("routes")]
public class RoutesController : ControllerBase
{
    [HttpPost]
    [SwaggerOperation("Add a route")]
    [SwaggerResponse(
        StatusCodes.Status201Created, "Object successfuly created",
        typeof(RouteDto))]
    [SwaggerResponse(
        StatusCodes.Status400BadRequest, "Input data validation error",
        typeof(HttpValidationProblemDetails))]
    [SwaggerResponse(
        StatusCodes.Status401Unauthorized, "Unauthorized to perform an action",
        typeof(ProblemDetails))]
    [SwaggerResponse(
        StatusCodes.Status403Forbidden,
        "Not enough privileges to perform an action",
        typeof(ProblemDetails))]
    [SwaggerResponse(
        StatusCodes.Status404NotFound, "One or more addresses was not found",
        typeof(ProblemDetails))]
    [SwaggerResponse(
        StatusCodes.Status500InternalServerError, "Internal server error",
        typeof(ProblemDetails))]
    public async Task<ActionResult<RouteDto>> Add(
        [FromBody] AddRouteViewModel viewModel,
        CancellationToken cancellationToken)
    {
        return StatusCode(
            StatusCodes.Status201Created,
            await Mediator.Send(
                new AddRouteCommand()
                {
                    Name = viewModel.Name,
                    VehicleType = VehicleType.FromName(viewModel.VehicleType),
                    Addresses = viewModel.Addresses.Select(
                        e => new RouteAddressModel()
                        {
                            Order = e.Order,
                            Guid = e.Uuid
                           
                        }).ToArray()
                },
                cancellationToken));
    }

    [HttpGet]
    [SwaggerOperation("Get a list of all routes")]
    [SwaggerResponse(
        StatusCodes.Status200OK, "Request successful",
        typeof(PaginatedList<RouteDto>))]
    [SwaggerResponse(
        StatusCodes.Status400BadRequest, "Input data validation error",
        typeof(HttpValidationProblemDetails))]
    [SwaggerResponse(
        StatusCodes.Status401Unauthorized, "Unauthorized to perform an action",
        typeof(ProblemDetails))]
    [SwaggerResponse(
        StatusCodes.Status403Forbidden,
        "Not enough privileges to perform an action",
        typeof(ProblemDetails))]
    [SwaggerResponse(
        StatusCodes.Status500InternalServerError, "Internal server error",
        typeof(ProblemDetails))]
    public async Task<PaginatedList<RouteDto>> GetPage(
        [FromQuery] PageQuery pageQuery, [FromQuery] SearchQuery searchQuery,
        [FromQuery] SortQuery sortQuery, 
        [FromQuery] GetRoutesPageFilterViewModel filterQuery,
        CancellationToken cancellationToken)
    {
        return await Mediator.Send(
            new GetRoutesPageQuery()
            {
                PageNumber = pageQuery.PageNumber,
                PageSize = pageQuery.PageSize,
                Search = searchQuery.Search,
                Sort = sortQuery.Sort,
                VehicleType = VehicleType.FromName(filterQuery.VehicleType)
            },
            cancellationToken);
    }

    [HttpGet("{uuid:guid}")]
    [SwaggerOperation("Get a route by uuid")]
    [SwaggerResponse(
        StatusCodes.Status200OK, "Request successful", typeof(RouteDto))]
    [SwaggerResponse(
        StatusCodes.Status400BadRequest, "Input data validation error",
        typeof(HttpValidationProblemDetails))]
    [SwaggerResponse(
        StatusCodes.Status401Unauthorized, "Unauthorized to perform an action",
        typeof(ProblemDetails))]
    [SwaggerResponse(
        StatusCodes.Status403Forbidden,
        "Not enough privileges to perform an action",
        typeof(ProblemDetails))]
    [SwaggerResponse(
        StatusCodes.Status404NotFound, "Object not found", typeof(RouteDto))]
    [SwaggerResponse(
        StatusCodes.Status500InternalServerError, "Internal server error",
        typeof(ProblemDetails))]
    public async Task<RouteDto> Get(
        [FromRoute] Guid uuid,
        CancellationToken cancellationToken)
    {
        return await Mediator.Send(new GetRouteQuery() { Guid = uuid },
            cancellationToken);
    }

    [HttpPut("{uuid:guid}")]
    [SwaggerOperation("Update a route")]
    [SwaggerResponse(
        StatusCodes.Status400BadRequest, "Object already exists",
        typeof(ProblemDetails))]
    [SwaggerResponse(
        StatusCodes.Status400BadRequest, "Input data validation error",
        typeof(HttpValidationProblemDetails))]
    [SwaggerResponse(
        StatusCodes.Status401Unauthorized, "Unauthorized to perform an action",
        typeof(ProblemDetails))]
    [SwaggerResponse(
        StatusCodes.Status403Forbidden,
        "Not enough privileges to perform an action",
        typeof(ProblemDetails))]
    [SwaggerResponse(
        StatusCodes.Status404NotFound, "Object not found", typeof(RouteDto))]
    [SwaggerResponse(
        StatusCodes.Status404NotFound, "One or more addresses was not found",
        typeof(ProblemDetails))]
    [SwaggerResponse(
        StatusCodes.Status500InternalServerError, "Internal server error",
        typeof(ProblemDetails))]
    public async Task<RouteDto> Update(
        [FromRoute] Guid uuid,
        [FromBody] UpdateRouteViewModel viewModel,
        CancellationToken cancellationToken)
    {
            return await Mediator.Send(
                new UpdateRouteCommand()
                {
                    Guid = uuid,
                    Name = viewModel.Name,
                    VehicleType = VehicleType.FromName(viewModel.VehicleType),
                    Addresses = viewModel.Addresses.Select(
                        e => new RouteAddressModel()
                        {
                            Order = e.Order,
                            Guid = e.Uuid
                           
                        }).ToArray()
                },
                cancellationToken);
    }

    [HttpDelete("{uuid:guid}")]
    [SwaggerOperation("Delete a route")]
    [SwaggerResponse(StatusCodes.Status204NoContent, "Request successful")]
    [SwaggerResponse(
        StatusCodes.Status400BadRequest, "Input data validation error",
        typeof(HttpValidationProblemDetails))]
    [SwaggerResponse(
        StatusCodes.Status401Unauthorized, "Unauthorized to perform an action",
        typeof(ProblemDetails))]
    [SwaggerResponse(
        StatusCodes.Status403Forbidden,
        "Not enough privileges to perform an action",
        typeof(ProblemDetails))]
    [SwaggerResponse(
        StatusCodes.Status404NotFound, "Object not found", typeof(RouteDto))]
    [SwaggerResponse(
        StatusCodes.Status500InternalServerError, "Internal server error",
        typeof(ProblemDetails))]
    public async Task<IActionResult> Delete(
        [FromRoute] Guid uuid,
        CancellationToken cancellationToken)
    {
        await Mediator.Send(
            new DeleteRouteCommand() { Guid = uuid },
            cancellationToken);
        return StatusCode(StatusCodes.Status204NoContent);
    }
}
